﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' Drawing constants.
    Private Const Xmin As Single = -0.1F
    Private Const Xmax As Single = 4.1F
    Private Const Ymin As Single = -10.0F
    Private Const Ymax As Single = 100.0F
    Private DrawingTransform, InverseTransform As Matrix

    Private HasSolution As Boolean = False
    Private BestA, BestB, BestC As Double
    Private Points As New List(Of PointF)()

    ' Make a drawing transformation.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim world_rect As New RectangleF(Xmin, Ymin, Xmax - Xmin, Ymax - Ymin)
        Dim pts() As PointF = _
        { _
            New PointF(0, picGraph.ClientSize.Height), _
            New PointF(picGraph.ClientSize.Width, picGraph.ClientSize.Height), _
            New PointF(0, 0) _
        }
        DrawingTransform = New Matrix(world_rect, pts)
        InverseTransform = DrawingTransform.Clone()
        InverseTransform.Invert()

        ' Pretty up the sample equation.
        rchEquation.Select(9, 2)
        rchEquation.SelectionCharOffset = 15
    End Sub

    ' Save a new point.
    Private Sub picGraph_MouseClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles picGraph.MouseClick
        ' Transform the point to world coordinates.
        Dim pts() As PointF = {New PointF(e.X, e.Y)}
        InverseTransform.TransformPoints(pts)
        ' Debug.WriteLine(pts(0).ToString())

        ' Save the point.
        Points.Add(pts(0))
        picGraph.Refresh()
    End Sub

    ' Draw the points and best fit curve.
    Private Sub picGraph_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles picGraph.Paint
        ' Use the drawing transformation.
        e.Graphics.Transform = DrawingTransform

        ' Draw the axes.
        DrawAxes(e.Graphics)

        ' Draw the curve.
        If (HasSolution) Then
            Const Xstep As Double = 0.01
            Dim x0 As Double = Xmin
            Dim y0 As Double = CurveFunctions.F(Xmin, BestA, BestB, BestC)
            Using thin_pen As New Pen(Color.Blue, 0)
                For x As Double = Xmin + Xstep To Xmax Step Xstep
                    Dim y1 As Double = F(x, BestA, BestB, BestC)
                    e.Graphics.DrawLine(thin_pen, _
                        CSng(x0), CSng(y0), CSng(x), CSng(y1))
                    x0 = x
                    y0 = y1
                    If (y0 > 100000000.0) Then Exit For
                Next x
            End Using
        End If

        ' Draw the points.
        Const dx As Single = (Xmax - Xmin) / 100
        Const dy As Single = (Ymax - Ymin) / 100
        Using thin_pen As New Pen(Color.Black, 0)
            For Each pt As PointF In Points
                e.Graphics.FillRectangle(Brushes.White, _
                    pt.X - dx, pt.Y - dy, 2 * dx, 2 * dy)
                e.Graphics.DrawRectangle(thin_pen, _
                    pt.X - dx, pt.Y - dy, 2 * dx, 2 * dy)
            Next pt
        End Using
    End Sub

    ' Draw the axes.
    Private Sub DrawAxes(ByVal gr As Graphics)
        Using thin_pen As New Pen(Color.Black, 0)
            Const xthick As Single = 0.05F
            Const ythick As Single = 2.0F

            gr.DrawLine(thin_pen, Xmin, 0, Xmax, 0)
            For x As Single = 0.5 To Xmax Step 0.5F
                gr.DrawLine(thin_pen, x, -ythick, x, ythick)
            Next x

            gr.DrawLine(thin_pen, 0, Ymin, 0, Ymax)
            For y As Single = 10.0F To Ymax Step 10.0F
                gr.DrawLine(thin_pen, -xthick, y, xthick, y)
            Next y
        End Using
    End Sub

    ' Clear the points.
    Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click
        Points = New List(Of PointF)()
        HasSolution = False
        picGraph.Refresh()

        txtA.Clear()
        txtB.Clear()
        txtC.Clear()
        txtError.Clear()
        txtPercentError.Clear()
    End Sub

    ' Find parameters for a weibull curve fit.
    Private Sub btnFit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnFit.Click
        Me.Cursor = Cursors.WaitCursor
        txtA.Clear()
        txtB.Clear()
        txtC.Clear()
        txtError.Clear()
        txtPercentError.Clear()
        Application.DoEvents()
        Dim start_time As Date = Date.Now

        ' Find a good fit.
        ' The final parameter, num_steps, determines how many initial
        ' values are tested for a, b, c, and d. For example, if num_steps = 4,
        ' then a, b, c, and d each take on 4 values for 4 ^ 4 = 256 initial points.
        Dim err As Double = FindGoodFit( _
            Points, BestA, BestB, BestC, 10, 100)

        Dim stop_time As Date = Date.Now
        Dim elapsed As TimeSpan = stop_time - start_time
        Console.WriteLine("Time: " & _
            elapsed.TotalSeconds.ToString("0.00") & " seconds")

        txtA.Text = BestA.ToString()
        txtB.Text = BestB.ToString()
        txtC.Text = BestC.ToString()

        ' Display the error.
        ShowError()

        ' We have a solution.
        HasSolution = True
        picGraph.Refresh()

        Me.Cursor = Cursors.Default
    End Sub

    ' Regraph with the given parameters.
    Private Sub btnGraph_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGraph.Click
        BestA = Double.Parse(txtA.Text)
        BestB = Double.Parse(txtB.Text)
        BestC = Double.Parse(txtC.Text)
        ShowError()
        picGraph.Refresh()
    End Sub

    ' Display the error.
    Private Sub ShowError()
        ' Get the error.
        Dim err As Double = Math.Sqrt(ErrorSquared( _
          Points, BestA, BestB, BestC))
        txtError.Text = err.ToString()

        ' Get the total of the function's values to
        ' calculate a percentage.
        Dim total As Double = 0
        For Each pt As PointF In Points
            total += F(pt.X, BestA, BestB, BestC)
        Next pt
        total = err / total * 100
        txtPercentError.Text = total.ToString()
    End Sub
End Class
